package com.dsic.gj_erp.wsclient;

import cn.hutool.cron.CronUtil;
import cn.hutool.cron.task.Task;
import com.alibaba.fastjson.JSONObject;
import com.dsic.gj_erp.wsclient.handler.TipHandler;
import com.dsic.gj_erp.wsclient.handler.ZxHandler;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.client.WebSocketClient;
import org.springframework.web.socket.client.WebSocketConnectionManager;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import javax.annotation.PostConstruct;
import java.io.IOException;

@Slf4j
@Service
@RequiredArgsConstructor
public class WebSocketService extends TextWebSocketHandler {

    @Getter
    private WebSocketSession session;

    private final WebSocketClient client;

    private final ZxHandler zxHandler;
    private final TipHandler tipHandler;

    @Getter
    @Value("${ws.client.url}")
    private String url;

    @Getter
    @Value("${ws.client.id}")
    private String erpId;

    @PostConstruct
    public void init() {
        System.out.println("========开始连接语音WS服务器========");
        this.connect(this.url);
    }

    public void connect(String uri) {
        try {
            WebSocketConnectionManager manager = new WebSocketConnectionManager(client, this, uri);
            manager.start();
        }catch (Exception e) {
            log.error("无法连接语音服务器:{},{}",this.url, e.getMessage());
        }
    }

    public void disconnect() throws Exception {
        if (session != null && session.isOpen()) {
            session.close();
        }
    }

    public void sendMessage(String message) throws Exception {
        if (session != null && session.isOpen()) {
            session.sendMessage(new TextMessage(message));
        } else {
            throw new IllegalStateException("WebSocket is not connected.");
        }
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        this.session = session; // 处理连接后保存session
        this.sendMessage(Message.reg(this.erpId).toString());
        this.heart(Message.heart(this.erpId));
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) throws IOException {
        if (this.session!=null){
            this.session.close();
            this.session=null;
        }
    }

    private boolean scheduleStartFlag=false;

    public void heart(Message message) {
        synchronized (this){
            if (this.scheduleStartFlag){
                return;
            }
            this.scheduleStartFlag=true;
        }
        CronUtil.schedule("*/15 * * * * ?", (Task) () -> {
            try {
                System.out.println("心跳");
                this.sendMessage(message.toString());
            } catch (Exception e) {
                log.error("语音服务器连接失效");
                this.connect(this.url);
            }
        });

        // 支持秒级别定时任务
        CronUtil.setMatchSecond(true);
        if (!CronUtil.getScheduler().isStarted()){
            CronUtil.start();
        }
    }

    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
        log.info("语音上报:{}",message.getPayload());
        Message msg= JSONObject.parseObject(message.getPayload(),Message.class);
        switch (msg.getCodeEx()){
            case REG:
                log.info("设备{},连接成功",msg.getMsg().getDeviceId());
                break;
            case TIP:
                tipHandler.execute(msg);
                break;
            case ERROR:
                tipHandler.execute(msg);
            case ZX:
                tipHandler.execute(msg);
                zxHandler.execute(msg);
                break;
        }
    }

}
